@@ -3,7 +3,7 @@
 
 use super::cssdest::CssDestination;
 use super::CssData;
-use crate::css::{self, AtRule, Import};
+use crate::css::{self, AtRule, Import, Selectors};
 use crate::error::ResultPos;
 use crate::input::{Context, Loader, Parsed, SourceKind};
 use crate::sass::{get_global_module, Expose, Item, UseAs};
@@ -67,8 +67,8 @@ fn handle_item(
                             if module.get_or_none(name).is_none() {
                                 module.define(name.clone(), value)?;
                             } else {
-                                return Err(Error::error(
-                                    "The same variable may only be configured once.",
+                                return Err(Error::S(
+                                    "The same variable may only be configured once.".to_string(),
                                 ));
                             }
                         }
@@ -117,8 +117,8 @@ fn handle_item(
                             if module.get_or_none(name).is_none() {
                                 module.define(name.clone(), value)?;
                             } else {
-                                return Err(Error::error(
-                                    "The same variable may only be configured once.",
+                                return Err(Error::S(
+                                    "The same variable may only be configured once.".to_string(),
                                 ));
                             }
                         }
@@ -133,7 +133,7 @@ fn handle_item(
                 file_context.unlock_loading(&sourcefile);
                 module?
             } else {
-                return Err(Error::S(format!("Module {} not found", name)));
+                return Err(Error::S(format!("Module {name} not found")));
             };
             scope.forward().do_use(module, &name, as_n, expose)?;
         }
@@ -178,7 +178,7 @@ fn handle_item(
                                 )?;
                             }
                             Parsed::Css(items) => {
-                                push_items(dest, items).at(pos)?
+                                push_items(dest, items).at(pos)?;
                             }
                         }
                         file_context.unlock_loading(&sourcefile);
@@ -213,6 +213,14 @@ fn handle_item(
                 handle_body(body, dest, subscope, file_context)?;
             }
         }
+        Item::AtMedia { args, body, pos: _ } => {
+            let args = args.evaluate(scope.clone())?;
+            let mut atmedia = dest.start_atmedia(args.try_into()?);
+            if let Some(ref body) = *body {
+                let local = ScopeRef::sub(scope);
+                handle_body(body, &mut atmedia, local, file_context)?;
+            }
+        }
         Item::AtRule {
             name,
             args,
@@ -222,8 +230,12 @@ fn handle_item(
             let name = name.evaluate(scope.clone())?.take_value();
             let args = args.evaluate(scope.clone())?;
             if let Some(ref body) = *body {
-                let mut atrule = dest.start_atrule(name, args);
-                let local = ScopeRef::sub(scope);
+                let mut atrule = dest.start_atrule(name.clone(), args);
+                let local = if name == "keyframes" {
+                    ScopeRef::sub_selectors(scope, Selectors::root())
+                } else {
+                    ScopeRef::sub(scope)
+                };
                 handle_body(body, &mut atrule, local, file_context)?;
             } else {
                 dest.push_item(AtRule::new(name, args, None).into())
@@ -252,7 +264,7 @@ fn handle_item(
 
         Item::MixinDeclaration(ref name, ref body) => {
             check_body(&body.body, BodyContext::Mixin)?;
-            scope.define_mixin(name.into(), body.closure(&scope).into())
+            scope.define_mixin(name.into(), body.closure(&scope).into());
         }
         Item::MixinCall(ref name, ref args, ref body, ref pos) => {
             if let Some(mixin) = scope.get_mixin(&name.into()) {
@@ -268,7 +280,7 @@ fn handle_item(
                         }
                         e => {
                             let pos = pos.in_call(name);
-                            Error::BadCall(format!("{:?}", e), pos, None)
+                            Error::BadCall(format!("{e:?}"), pos, None)
                         }
                     })?;
             } else {
@@ -331,15 +343,18 @@ fn handle_item(
         }
 
         Item::Debug(ref value) => {
-            eprintln!("DEBUG: {}", value.evaluate(scope)?.introspect())
+            eprintln!("DEBUG: {}", value.evaluate(scope)?.introspect());
         }
         Item::Warn(ref value) => {
             eprintln!("WARNING: {}", value.evaluate(scope)?.introspect());
         }
         Item::Error(ref value, ref pos) => {
-            let msg = value.evaluate(scope)?.introspect().to_string();
+            let msg = value.evaluate(scope)?.introspect();
             return Err(Invalid::AtError(msg).at(pos.clone()));
         }
+        Item::Extend(_selectors) => {
+            return Err(Error::S("@extend is not supported yet".to_string()));
+        }
 
         Item::Rule(ref selectors, ref body) => {
             check_body(body, BodyContext::Rule)?;
@@ -444,8 +459,7 @@ fn check_body(body: &[Item], context: BodyContext) -> Result<(), Error> {
             } if context != BodyContext::Rule => {
                 if !name
                     .single_raw()
-                    .map(|name| CSS_AT_RULES.contains(&name))
-                    .unwrap_or(false)
+                    .map_or(false, |name| CSS_AT_RULES.contains(&name))
                 {
                     return Err(Invalid::AtRule.at(pos.clone()));
                 }
