@@ -3,6 +3,7 @@ mod css_function;
 mod error;
 pub mod formalargs;
 mod imports;
+mod media;
 pub mod selectors;
 mod span;
 pub(crate) mod strings;
@@ -167,7 +168,7 @@ fn at_root2(input: Span) -> PResult<Item> {
 fn mixin_call2(input: Span) -> PResult<Item> {
     let (rest, n1) = terminated(name, opt_spacelike)(input)?;
     let (rest, n2) = opt(preceded(tag("."), name))(rest)?;
-    let name = n2.map(|n2| format!("{}.{}", n1, n2)).unwrap_or(n1);
+    let name = n2.map(|n2| format!("{n1}.{n2}")).unwrap_or(n1);
     let (rest, _) = opt_spacelike(rest)?;
     let (rest0, args) = terminated(opt(call_args), opt_spacelike)(rest)?;
     let (rest, t) = alt((tag("using"), tag("{"), tag("")))(rest0)?;
@@ -197,8 +198,8 @@ fn mixin_call2(input: Span) -> PResult<Item> {
 
 /// What follows an `@` sign
 fn at_rule2(input0: Span) -> PResult<Item> {
-    let (input, name) = terminated(name, opt_spacelike)(input0)?;
-    match name.as_ref() {
+    let (input, name) = terminated(sass_string, opt_spacelike)(input0)?;
+    match name.single_raw().unwrap_or("") {
         "at-root" => at_root2(input),
         "charset" => charset2(input),
         "content" => content_stmt2(input),
@@ -210,21 +211,42 @@ fn at_rule2(input0: Span) -> PResult<Item> {
             let pos = input0.up_to(&end).to_owned().opt_back("@");
             Ok((rest, Item::Error(v, pos)))
         }
+        "extend" => map(
+            delimited(
+                opt_spacelike,
+                selectors,
+                preceded(opt_spacelike, tag(";")),
+            ),
+            Item::Extend,
+        )(input),
         "for" => for_loop2(input),
         "forward" => forward2(input0),
         "function" => function_declaration2(input),
         "if" => if_statement2(input),
         "import" => import2(input),
         "include" => mixin_call2(input),
+        "media" => media::rule(input0),
         "mixin" => mixin_declaration2(input),
         "return" => return_stmt2(input0, input),
         "use" => use2(input0),
         "warn" => map(expression_argument, Item::Warn)(input),
         "while" => while_loop2(input),
         _ => {
-            let (input, name) = sass_string(input0)?;
             let pos = input0.up_to(&input).to_owned().opt_back("@");
-            let (input, args) = opt(media_args)(input)?;
+            let (input, args) = opt(unknown_rule_args)(input)?;
+            fn x_args(value: Value) -> Value {
+                match value {
+                    Value::Variable(name, _pos) => {
+                        Value::Literal(SassString::from(format!("${name}")))
+                    }
+                    Value::Map(map) => Value::Map(
+                        map.into_iter()
+                            .map(|(k, v)| (x_args(k), x_args(v)))
+                            .collect(),
+                    ),
+                    value => value,
+                }
+            }
             let (input, body) = preceded(
                 opt(ignore_space),
                 alt((map(body_block, Some), value(None, semi_or_end))),
@@ -233,7 +255,7 @@ fn at_rule2(input0: Span) -> PResult<Item> {
                 input,
                 Item::AtRule {
                     name,
-                    args: args.unwrap_or(Value::Null),
+                    args: args.map_or(Value::Null, x_args),
                     body,
                     pos,
                 },
@@ -265,7 +287,8 @@ fn charset2(input: Span) -> PResult<Item> {
     )(input)
 }
 
-fn media_args(input: Span) -> PResult<Value> {
+/// Arguments to an unkown at rule.
+fn unknown_rule_args(input: Span) -> PResult<Value> {
     let (input, args) = separated_list0(
         preceded(tag(","), opt_spacelike),
         map(
@@ -277,7 +300,7 @@ fn media_args(input: Span) -> PResult<Value> {
                             function_call,
                             dictionary,
                             map(
-                                delimited(tag("("), media_args, tag(")")),
+                                delimited(tag("("), media::args, tag(")")),
                                 |v| Value::Paren(Box::new(v), true),
                             ),
                             map(sass_string, Value::Literal),
@@ -289,7 +312,7 @@ fn media_args(input: Span) -> PResult<Value> {
                             value((), peek(one_of(") \r\n\t{,;"))),
                         )),
                     ),
-                    map(map_res(is_not("#()\"'{};, "), input_to_str), |s| {
+                    map(map_res(is_not("\"'{};#"), input_to_str), |s| {
                         Value::Literal(s.trim_end().into())
                     }),
                 )),
@@ -321,20 +344,6 @@ pub(crate) fn check_parse<T>(
     ParseError::check(parser(code_span(value).borrow()))
 }
 
-#[test]
-fn test_media_args_1() {
-    check_parse(media_args, b"#{$media} and ($key + \"-foo\": $value + 5)")
-        .unwrap();
-}
-#[test]
-fn test_media_args_2() {
-    check_parse(
-        media_args,
-        b"print and (foo: 1 2 3), (bar: 3px hux(muz)), not screen",
-    )
-    .unwrap();
-}
-
 fn if_statement_inner(input: Span) -> PResult<Item> {
     preceded(
         terminated(verify(name, |n: &String| n == "if"), opt_spacelike),
@@ -350,7 +359,7 @@ fn if_statement2(input: Span) -> PResult<Item> {
         name,
         opt_spacelike,
     ))(input)?;
-    match word.as_ref().map(|w| w.as_ref()) {
+    match word.as_ref().map(AsRef::as_ref) {
         Some("else") => {
             let (input2, else_body) = alt((
                 body_block,
@@ -456,6 +465,7 @@ fn content_stmt2(input: Span) -> PResult<Item> {
 fn custom_property(input: Span) -> PResult<Item> {
     let (rest, name) = terminated(opt(sass_string), tag(":"))(input)?;
     let mut name = name.unwrap_or_else(|| SassString::from(""));
+    // The dashes was parsed before calling this method.
     name.prepend("--");
     let (rest, value) =
         terminated(custom_value, alt((tag(";"), peek(tag("}")))))(rest)?;
