@@ -5,7 +5,7 @@ use super::strings::{
     special_function_misc, special_url, var_name,
 };
 use super::unit::unit;
-use super::util::{comment, ignore_comments, opt_spacelike, spacelike2};
+use super::util::{ignore_comments, opt_spacelike, spacelike2};
 use super::{input_to_string, position, sass_string, PResult, Span};
 use crate::sass::{BinOp, SassString, Value};
 use crate::value::{ListSeparator, Number, Numeric, Operator, Rgba};
@@ -14,7 +14,9 @@ use nom::bytes::complete::{tag, tag_no_case};
 use nom::character::complete::{
     alphanumeric1, multispace0, multispace1, one_of,
 };
-use nom::combinator::{into, map, map_res, not, opt, peek, recognize, value};
+use nom::combinator::{
+    into, map, map_res, not, opt, peek, recognize, value, verify,
+};
 use nom::multi::{fold_many0, fold_many1, many0, many_m_n, separated_list1};
 use nom::sequence::{delimited, pair, preceded, terminated, tuple};
 use num_traits::Zero;
@@ -119,18 +121,7 @@ fn logic_expression(input: Span) -> PResult<Value> {
     let (input1, a) = sum_expression(input)?;
     fold_many0(
         tuple((
-            delimited(
-                multispace0,
-                alt((
-                    value(Operator::Equal, tag("==")),
-                    value(Operator::NotEqual, tag("!=")),
-                    value(Operator::GreaterE, tag(">=")),
-                    value(Operator::Greater, tag(">")),
-                    value(Operator::LesserE, tag("<=")),
-                    value(Operator::Lesser, tag("<")),
-                )),
-                multispace0,
-            ),
+            delimited(multispace0, relational_operator, multispace0),
             sum_expression,
             position,
         )),
@@ -142,42 +133,59 @@ fn logic_expression(input: Span) -> PResult<Value> {
     )(input1)
 }
 
+fn relational_operator(input: Span) -> PResult<Operator> {
+    alt((
+        value(Operator::Equal, tag("==")),
+        value(Operator::NotEqual, tag("!=")),
+        value(Operator::GreaterE, tag(">=")),
+        value(Operator::Greater, tag(">")),
+        value(Operator::LesserE, tag("<=")),
+        value(Operator::Lesser, tag("<")),
+    ))(input)
+}
+
 fn sum_expression(input: Span) -> PResult<Value> {
-    let (rest, v) = term_value(input)?;
+    any_additive_expr(term_value, input)
+}
+
+pub fn any_additive_expr<F>(term: F, input: Span) -> PResult<Value>
+where
+    F: Fn(Span) -> PResult<Value>,
+{
+    let (rest, v) = term(input)?;
     fold_many0(
-        alt((
-            tuple((
-                map(opt(comment), |_| false),
-                alt((
-                    value(Operator::Plus, tag("+")),
-                    value(Operator::Minus, tag("-")),
+        tuple((
+            verify(
+                tuple((
+                    ignore_comments,
+                    alt((
+                        value(Operator::Plus, tag("+")),
+                        value(Operator::Minus, tag("-")),
+                    )),
+                    ignore_comments,
                 )),
-                ignore_comments,
-                term_value,
-                position,
-            )),
-            tuple((
-                ignore_comments,
-                alt((
-                    value(Operator::Plus, tag("+")),
-                    value(Operator::Minus, terminated(tag("-"), spacelike2)),
-                )),
-                ignore_comments,
-                term_value,
-                position,
-            )),
+                |(s1, op, s2)| op == &Operator::Plus || !*s1 || *s2,
+            ),
+            term,
+            position,
         )),
         move || v.clone(),
-        |v, (s1, op, s2, v2, end)| {
+        |v, ((s1, op, s2), v2, end)| {
             let pos = input.up_to(&end).to_owned();
-            let s2 = s2 || (s1 && op == Operator::Minus);
             BinOp::new(v, s1, op, s2, v2, pos).into()
         },
     )(rest)
 }
 
 fn term_value(input: Span) -> PResult<Value> {
-    let (rest, v) = single_value(input)?;
+    any_product(single_value, input)
+}
+
+pub fn any_product<F>(factor: F, input: Span) -> PResult<Value>
+where
+    F: Fn(Span) -> PResult<Value>,
+{
+    let (rest, v) = factor(input)?;
     fold_many0(
         tuple((
             ignore_comments,
@@ -190,7 +198,7 @@ fn term_value(input: Span) -> PResult<Value> {
                 value(Operator::Modulo, tag("%")),
             )),
             ignore_comments,
-            single_value,
+            factor,
             position,
         )),
         move || v.clone(),
@@ -305,7 +313,7 @@ fn unicode_range(input: Span) -> PResult<Value> {
     ))
 }
 
-fn bracket_list(input: Span) -> PResult<Value> {
+pub fn bracket_list(input: Span) -> PResult<Value> {
     let (input, content) =
         delimited(tag("["), opt(value_expression), tag("]"))(input)?;
     Ok((
@@ -504,7 +512,7 @@ pub fn dictionary_inner(input: Span) -> PResult<Value> {
             pair(
                 sum_expression,
                 preceded(
-                    delimited(opt_spacelike, tag(":"), opt_spacelike),
+                    delimited(ignore_comments, tag(":"), opt_spacelike),
                     space_list,
                 ),
             ),
