@@ -14,10 +14,14 @@ fn test() {
              \n.test11#{\'\\@#{$key}\'} { content: \'1.1\'; }\n"
         ),
         "Error: expected selector.\
-         \n  ,\
-         \n2 | .test11@bar{ content: \'1.1\'; }\
-         \n  |        ^\
+         \n  ,--> input.scss\
+         \n2 | .test11#{\'\\@#{$key}\'} { content: \'1.1\'; }\
+         \n  |          ^^^^^^^^^^^ \
          \n  \'\
-         \n  input.scss 2:8  root stylesheet",
+         \n  ,\
+         \n1 | .test11@bar \
+         \n  |        = error in interpolated output\
+         \n  \'\
+         \n  input.scss 2:10  root stylesheet",
     );
 }
